Package: cograph
Title: Analysis and Visualization of Complex Networks
Version: 2.4.4
Authors@R: c(
    person("Mohammed", "Saqr", role = c("aut", "cph")),
    person("Sonsoles", "L\u00f3pez-Pernas",
           email = "sonsoles.lopez@uef.fi", role = c("aut", "cre", "cph"))
  )
Author: Mohammed Saqr [aut, cph],
  Sonsoles López-Pernas [aut, cre, cph]
Maintainer: Sonsoles López-Pernas <sonsoles.lopez@uef.fi>
Description: Provides tools for the analysis, visualization, and manipulation
  of dynamical, social (Saqr et al. (2024) <doi:10.1007/978-3-031-54464-4_10>) and 
  complex networks (Saqr et al. (2025) <doi:10.1145/3706468.3706513>). The package 
  supports multiple network formats and offers flexible tools for heterogeneous, 
  multi-layer, and hierarchical network analysis with simple syntax and
  extensive toolset.
License: MIT + file LICENSE
URL: https://sonsoles.me/cograph/,
        https://github.com/sonsoleslp/cograph
BugReports: https://github.com/sonsoleslp/cograph/issues
Depends: R (>= 4.1.0)
Imports: ggplot2 (>= 3.4.0), grDevices, grid, Matrix, R6, stats, utils
Suggests: backbone, brainGraph, centiserve, colorspace, digest, dplyr,
        gifski, gridExtra, grImport2, igraph, influenceR, jsonlite,
        knitr, Nestimate, network, qgraph, RColorBrewer, reticulate,
        rmarkdown, rsvg, scales, sna, testthat (>= 3.0.0), tidygraph,
        tna, tnet, viridisLite
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
LazyData: true
NeedsCompilation: no
Packaged: 2026-07-10 14:12:07 UTC; mohammedsaqr
Repository: CRAN
Date/Publication: 2026-07-10 16:10:02 UTC
