# Generated by roxygen2: do not edit by hand

S3method(as_mcml,cluster_summary)
S3method(as_mcml,default)
S3method(as_mcml,group_tna)
S3method(as_mcml,mcml)
S3method(as_tna,cluster_summary)
S3method(as_tna,default)
S3method(as_tna,mcml)
S3method(disparity_filter,cograph_network)
S3method(disparity_filter,default)
S3method(disparity_filter,igraph)
S3method(disparity_filter,matrix)
S3method(disparity_filter,tna)
S3method(plot,cograph_cluster_significance)
S3method(plot,cograph_communities)
S3method(plot,cograph_core_periphery)
S3method(plot,cograph_degree_fit)
S3method(plot,cograph_motif_analysis)
S3method(plot,cograph_motif_result)
S3method(plot,cograph_motifs)
S3method(plot,cograph_network)
S3method(plot,cograph_rich_club)
S3method(plot,cograph_vulnerability)
S3method(plot,net_bootstrap_group)
S3method(plot,netobject_group)
S3method(plot,netobject_ml)
S3method(plot,tna_bootstrap)
S3method(plot,tna_disparity)
S3method(plot_bootstrap_forest,boot_glasso)
S3method(plot_bootstrap_forest,net_bootstrap)
S3method(plot_bootstrap_forest,net_bootstrap_group)
S3method(plot_bootstrap_forest,tna_bootstrap)
S3method(plot_edge_diff_forest,boot_glasso)
S3method(print,cluster_quality)
S3method(print,cluster_summary)
S3method(print,cograph_assortativity)
S3method(print,cograph_cluster_significance)
S3method(print,cograph_communities)
S3method(print,cograph_core_periphery)
S3method(print,cograph_degree_fit)
S3method(print,cograph_dyad_census)
S3method(print,cograph_ego_networks)
S3method(print,cograph_k_paths)
S3method(print,cograph_motif_analysis)
S3method(print,cograph_motif_result)
S3method(print,cograph_motifs)
S3method(print,cograph_network)
S3method(print,cograph_rich_club)
S3method(print,cograph_vulnerability)
S3method(print,mcml)
S3method(print,tna_disparity)
S3method(simplify,cograph_network)
S3method(simplify,default)
S3method(simplify,igraph)
S3method(simplify,matrix)
S3method(simplify,tna)
S3method(summary,cograph_network)
export(CographLayout)
export(CographNetwork)
export(CographTheme)
export(abbrev_label)
export(aggregate_layers)
export(aggregate_weights)
export(as_cograph)
export(as_mcml)
export(as_tna)
export(assortativity)
export(assortativity_attribute)
export(centrality)
export(centrality_alpha)
export(centrality_authority)
export(centrality_average_distance)
export(centrality_barycenter)
export(centrality_betweenness)
export(centrality_bottleneck)
export(centrality_bridging)
export(centrality_brokerage_coordinator)
export(centrality_brokerage_gatekeeper)
export(centrality_brokerage_itinerant)
export(centrality_brokerage_liaison)
export(centrality_brokerage_representative)
export(centrality_centroid)
export(centrality_closeness)
export(centrality_closeness_vitality)
export(centrality_clusterrank)
export(centrality_communicability)
export(centrality_communicability_betweenness)
export(centrality_constraint)
export(centrality_coreness)
export(centrality_cross_clique)
export(centrality_current_flow_betweenness)
export(centrality_current_flow_closeness)
export(centrality_dangalchev)
export(centrality_decay)
export(centrality_degree)
export(centrality_diffusion)
export(centrality_diversity)
export(centrality_dmnc)
export(centrality_eccentricity)
export(centrality_effective_size)
export(centrality_eigenvector)
export(centrality_entropy)
export(centrality_expected)
export(centrality_expected_influence_1)
export(centrality_expected_influence_2)
export(centrality_flow_betweenness)
export(centrality_gateway)
export(centrality_generalized_closeness)
export(centrality_gilschmidt)
export(centrality_harary)
export(centrality_harmonic)
export(centrality_hub)
export(centrality_hubbell)
export(centrality_incloseness)
export(centrality_indegree)
export(centrality_ineccentricity)
export(centrality_information)
export(centrality_inharmonic)
export(centrality_instrength)
export(centrality_integration)
export(centrality_katz)
export(centrality_kreach)
export(centrality_lac)
export(centrality_laplacian)
export(centrality_leaderrank)
export(centrality_leverage)
export(centrality_lin)
export(centrality_load)
export(centrality_lobby)
export(centrality_local_bridging)
export(centrality_markov)
export(centrality_mnc)
export(centrality_outcloseness)
export(centrality_outdegree)
export(centrality_outeccentricity)
export(centrality_outharmonic)
export(centrality_outstrength)
export(centrality_pagerank)
export(centrality_pairwisedis)
export(centrality_participation)
export(centrality_percolation)
export(centrality_power)
export(centrality_prestige_domain)
export(centrality_prestige_domain_proximity)
export(centrality_radiality)
export(centrality_random_walk)
export(centrality_reaching_local)
export(centrality_residual_closeness)
export(centrality_salsa)
export(centrality_semilocal)
export(centrality_strength)
export(centrality_stress)
export(centrality_subgraph)
export(centrality_topological_coefficient)
export(centrality_transitivity)
export(centrality_voterank)
export(centrality_wiener)
export(centrality_within_module_z)
export(centralization)
export(cluster_quality)
export(cluster_significance)
export(cnet)
export(cograph)
export(color_communities)
export(com_consensus)
export(com_eb)
export(com_fg)
export(com_fl)
export(com_im)
export(com_ld)
export(com_le)
export(com_lp)
export(com_lv)
export(com_op)
export(com_sg)
export(com_wt)
export(communities)
export(community_consensus)
export(community_edge_betweenness)
export(community_fast_greedy)
export(community_fluid)
export(community_infomap)
export(community_label_propagation)
export(community_leading_eigenvector)
export(community_leiden)
export(community_louvain)
export(community_optimal)
export(community_sizes)
export(community_spinglass)
export(community_walktrap)
export(compare_communities)
export(core_periphery)
export(cqual)
export(csig)
export(csum)
export(degree_distribution)
export(detect_communities)
export(disparity_filter)
export(dispersion)
export(dyad_census)
export(edge_betweenness)
export(edge_centrality)
export(edge_reciprocity)
export(ego_networks)
export(estrada_index)
export(extract_interlayer)
export(extract_layer)
export(extract_motifs)
export(extract_triads)
export(filter_edges)
export(filter_nodes)
export(fit_degree_distribution)
export(from_qgraph)
export(from_tna)
export(get_data)
export(get_edge_list)
export(get_edges)
export(get_groups)
export(get_labels)
export(get_layout)
export(get_meta)
export(get_nodes)
export(get_shape)
export(get_source)
export(get_theme)
export(ggplot_robustness)
export(group_centrality)
export(homophily)
export(htna)
export(is_bipartite)
export(is_directed)
export(is_tna_network)
export(k_shortest_paths)
export(label_abbrev)
export(lagg)
export(layer_degree_correlation)
export(layer_similarity)
export(layer_similarity_matrix)
export(layout_circle)
export(layout_groups)
export(layout_oval)
export(layout_saqr)
export(layout_spring)
export(layout_target)
export(ldegcor)
export(list_layouts)
export(list_palettes)
export(list_shapes)
export(list_svg_shapes)
export(list_themes)
export(lsim)
export(lsim_matrix)
export(mcml)
export(membership)
export(mlna)
export(motif_census)
export(motifs)
export(mtna)
export(n_communities)
export(n_edges)
export(n_nodes)
export(neighborhood_overlap)
export(network_bridges)
export(network_clique_size)
export(network_cut_vertices)
export(network_girth)
export(network_global_efficiency)
export(network_local_efficiency)
export(network_radius)
export(network_rich_club)
export(network_small_world)
export(network_summary)
export(network_vertex_connectivity)
export(nodes)
export(overlay_communities)
export(palette_blues)
export(palette_colorblind)
export(palette_diverging)
export(palette_pastel)
export(palette_rainbow)
export(palette_reds)
export(palette_viridis)
export(panel_layout)
export(plot_alluvial)
export(plot_bootstrap_forest)
export(plot_centrality)
export(plot_centrality_compare)
export(plot_centrality_distribution)
export(plot_centrality_heatmap)
export(plot_chord)
export(plot_compare)
export(plot_comparison_heatmap)
export(plot_degree_correlation)
export(plot_difference)
export(plot_edge_diff_forest)
export(plot_edge_weights)
export(plot_group_permutation)
export(plot_heatmap)
export(plot_htna)
export(plot_mcml)
export(plot_mixed_network)
export(plot_ml_heatmap)
export(plot_mlna)
export(plot_motifs)
export(plot_mtna)
export(plot_net_bootstrap_group)
export(plot_net_stability)
export(plot_netobject_group)
export(plot_netobject_ml)
export(plot_network_evolution)
export(plot_permutation)
export(plot_robustness)
export(plot_simplicial)
export(plot_temporal)
export(plot_tna)
export(plot_trajectories)
export(plot_transitions)
export(project_bipartite)
export(reaching_global)
export(register_layout)
export(register_shape)
export(register_svg_shape)
export(register_theme)
export(rich_club)
export(rich_club_local)
export(robustness)
export(robustness_auc)
export(robustness_summary)
export(select_bridges)
export(select_component)
export(select_edges)
export(select_edges_between)
export(select_edges_involving)
export(select_neighbors)
export(select_nodes)
export(select_top)
export(select_top_edges)
export(set_edges)
export(set_groups)
export(set_layout)
export(set_nodes)
export(shortest_paths)
export(simmelian_strength)
export(simplify)
export(sn_edges)
export(sn_ggplot)
export(sn_layout)
export(sn_nodes)
export(sn_palette)
export(sn_render)
export(sn_save)
export(sn_save_ggplot)
export(sn_theme)
export(soplot)
export(splot)
export(splot.boot_glasso)
export(splot.cograph_communities)
export(splot.group_tna_permutation)
export(splot.net_bootstrap)
export(splot.net_mlvar)
export(splot.net_permutation)
export(splot.netobject)
export(splot.tna_bootstrap)
export(splot.tna_communities)
export(splot.tna_disparity)
export(splot.tna_permutation)
export(splot.wtna_mixed)
export(subgraphs)
export(subset_edges)
export(subset_nodes)
export(summarize_clusters)
export(summarize_network)
export(supra)
export(supra_adjacency)
export(supra_interlayer)
export(supra_layer)
export(theme_cograph_classic)
export(theme_cograph_colorblind)
export(theme_cograph_dark)
export(theme_cograph_gray)
export(theme_cograph_minimal)
export(theme_cograph_nature)
export(theme_cograph_viridis)
export(to_cograph)
export(to_data_frame)
export(to_df)
export(to_igraph)
export(to_matrix)
export(to_network)
export(tplot)
export(triad_census)
export(trophic_incoherence)
export(unregister_svg_shape)
export(verify_igraph)
export(verify_with_igraph)
export(vulnerability)
export(wagg)
import(R6)
import(ggplot2)
import(grid)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(grDevices,svg)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(utils,modifyList)
