% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-globals.R
\name{list_shapes}
\alias{list_shapes}
\title{List Available Shapes}
\usage{
list_shapes()
}
\value{
Character vector of registered shape names.
}
\description{
List Available Shapes
}
\examples{
list_shapes()
}
