% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-distribution.R
\name{print.cograph_degree_fit}
\alias{print.cograph_degree_fit}
\title{Print method for cograph_degree_fit}
\usage{
\method{print}{cograph_degree_fit}(x, digits = 4, ...)
}
\arguments{
\item{x}{A \code{cograph_degree_fit} object from
\code{\link{fit_degree_distribution}}.}

\item{digits}{Number of decimal places. Default 4.}

\item{...}{Additional arguments passed to \code{print.data.frame}.}
}
\value{
Invisible \code{x}.
}
\description{
Displays the comparison table of fitted distributions sorted by AIC.
}
\examples{
adj <- matrix(c(0, 1, 1, 0, 0,
                1, 0, 1, 1, 0,
                1, 1, 0, 1, 1,
                0, 1, 1, 0, 1,
                0, 0, 1, 1, 0), 5, 5, byrow = TRUE)
fit <- cograph::fit_degree_distribution(adj,
  distributions = c("exponential", "poisson"))
print(fit)
}
