% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout-groups.R
\name{layout-groups}
\alias{layout-groups}
\title{Group-based Layout}
\description{
Arrange nodes in groups, with each group in a circular arrangement.
}
\keyword{internal}
