% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster-metrics.R
\name{layer_similarity_matrix}
\alias{layer_similarity_matrix}
\alias{lsim_matrix}
\title{Pairwise Layer Similarities}
\usage{
layer_similarity_matrix(
  layers,
  method = c("jaccard", "overlap", "cosine", "pearson")
)

lsim_matrix(layers, method = c("jaccard", "overlap", "cosine", "pearson"))
}
\arguments{
\item{layers}{List of adjacency matrices (one per layer)}

\item{method}{Similarity method}
}
\value{
Symmetric matrix of pairwise similarities
}
\description{
Computes similarity matrix for all pairs of layers.
}
\examples{
nodes <- c("A", "B", "C")
t1 <- matrix(c(0, 1, 0, 1, 0, 1, 0, 1, 0), 3, 3, dimnames = list(nodes, nodes))
t2 <- matrix(c(0, 1, 1, 1, 0, 0, 1, 0, 0), 3, 3, dimnames = list(nodes, nodes))
layers <- list(T1 = t1, T2 = t2)

layer_similarity_matrix(layers, "cosine")
layer_similarity_matrix(layers, "jaccard")
}
