% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input-edgelist.R
\name{input-edgelist}
\alias{input-edgelist}
\title{Edge List Input Parsing}
\description{
Functions for parsing edge list data frames.
}
\keyword{internal}
