% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-plot.R
\name{methods-plot}
\alias{methods-plot}
\title{Plot Methods}
\description{
S3 plot methods for Cograph objects.
}
\keyword{internal}
