% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aes-edges.R
\name{aes-edges}
\alias{aes-edges}
\title{Edge Aesthetics}
\description{
Functions for setting edge aesthetic properties.
}
\keyword{internal}
