% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aes-nodes.R
\name{aes-nodes}
\alias{aes-nodes}
\title{Node Aesthetics}
\description{
Functions for setting node aesthetic properties.
}
\keyword{internal}
