% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes-registry.R
\name{themes-registry}
\alias{themes-registry}
\title{Theme Registry Functions}
\value{
No return value, called for side effects.
}
\description{
Functions for registering built-in themes.
}
\keyword{internal}
