% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input-tna.R
\name{input-tna}
\alias{input-tna}
\title{tna Input Parsing}
\description{
Functions for parsing tna objects.
}
\keyword{internal}
