% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes-builtin.R
\name{theme_cograph_dark}
\alias{theme_cograph_dark}
\title{Dark Theme}
\usage{
theme_cograph_dark()
}
\value{
A CographTheme object.
}
\description{
Dark background theme for presentations.
}
\examples{
theme <- theme_cograph_dark()
}
