% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input-matrix.R
\name{input-matrix}
\alias{input-matrix}
\title{Matrix Input Parsing}
\description{
Functions for parsing adjacency/weight matrices.
}
\keyword{internal}
