% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-compare.R
\name{plot_comparison_heatmap}
\alias{plot_comparison_heatmap}
\title{Plot Comparison Heatmap}
\usage{
plot_comparison_heatmap(
  x,
  y = NULL,
  type = c("difference", "x", "y"),
  name_x = "x",
  name_y = "y",
  low_color = "blue",
  mid_color = "white",
  high_color = "red",
  limits = NULL,
  show_values = FALSE,
  value_size = 3,
  digits = 2,
  title = NULL,
  xlab = "Target",
  ylab = "Source"
)
}
\arguments{
\item{x}{First network: matrix, \code{cograph_network},
\code{CographNetwork}, \code{tna}, \code{igraph}, or list-like object
with \code{$weights}.}

\item{y}{Second network: same type as x. NULL to plot just x.}

\item{type}{What to display: "difference" (x - y), "x", or "y".}

\item{name_x}{Label for first network in title. Default "x".}

\item{name_y}{Label for second network in title. Default "y".}

\item{low_color}{Color for low/negative values. Default "blue".}

\item{mid_color}{Color for zero/middle values. Default "white".}

\item{high_color}{Color for high/positive values. Default "red".}

\item{limits}{Color scale limits. NULL for auto. Use c(-1, 1) for normalized.}

\item{show_values}{Logical: display values in cells? Default FALSE.}

\item{value_size}{Text size for cell values. Default 3.}

\item{digits}{Decimal places for cell values. Default 2.}

\item{title}{Plot title. NULL for auto-generated.}

\item{xlab}{X-axis label. Default "Target".}

\item{ylab}{Y-axis label. Default "Source".}
}
\value{
A ggplot2 object.
}
\description{
Creates a heatmap visualization comparing two networks.
}
\examples{
set.seed(42)
m1 <- matrix(runif(25), 5, 5)
m2 <- matrix(runif(25), 5, 5)
rownames(m1) <- colnames(m1) <- LETTERS[1:5]
rownames(m2) <- colnames(m2) <- LETTERS[1:5]
plot_comparison_heatmap(m1, m2)
plot_comparison_heatmap(m1, type = "x")

}
