% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input-igraph.R
\name{input-igraph}
\alias{input-igraph}
\title{igraph Input Parsing}
\description{
Functions for parsing igraph objects.
}
\keyword{internal}
