% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cograph.R
\name{cograph-main}
\alias{cograph-main}
\title{Main Entry Point}
\description{
The primary function for creating network visualizations.
}
\keyword{internal}
