% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes-builtin.R
\name{theme_cograph_viridis}
\alias{theme_cograph_viridis}
\title{Viridis Theme}
\usage{
theme_cograph_viridis()
}
\value{
A CographTheme object.
}
\description{
Theme using viridis color palette.
}
\examples{
theme <- theme_cograph_viridis()
}
