% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-network.R
\name{n_nodes}
\alias{n_nodes}
\title{Get Number of Nodes}
\usage{
n_nodes(x)
}
\arguments{
\item{x}{A cograph_network object.}
}
\value{
Integer: number of nodes.
}
\description{
Returns the number of nodes in a cograph_network.
}
\examples{
mat <- matrix(c(0, 1, 1, 1, 0, 1, 1, 1, 0), nrow = 3)
net <- as_cograph(mat)
n_nodes(net)  # 3
}
\seealso{
\code{\link{as_cograph}}, \code{\link{n_edges}}, \code{\link{get_nodes}}
}
