% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-plot.R
\name{plot.cograph_network}
\alias{plot.cograph_network}
\title{Plot cograph_network Object}
\usage{
\method{plot}{cograph_network}(x, ...)
}
\arguments{
\item{x}{A cograph_network object.}

\item{...}{Additional arguments passed to sn_render.}
}
\value{
The input object \code{x}, invisibly.
}
\description{
Plot cograph_network Object
}
\examples{
adj <- matrix(c(0, 1, 1, 1, 0, 1, 1, 1, 0), nrow = 3)
net <- cograph(adj)
plot(net)

}
