% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-network.R
\name{get_groups}
\alias{get_groups}
\title{Get Node Groups from Cograph Network}
\usage{
get_groups(x)
}
\arguments{
\item{x}{A cograph_network object.}
}
\value{
A data frame with node groupings, or NULL if not set. The data frame
has columns:
\itemize{
\item \code{node}: Node labels
\item One of \code{layer}, \code{cluster}, or \code{group}: Group assignment
}
}
\description{
Extracts the node groupings from a cograph_network object.
}
\examples{
mat <- matrix(runif(25), 5, 5)
rownames(mat) <- colnames(mat) <- LETTERS[1:5]
net <- as_cograph(mat)
net <- set_groups(net, list(G1 = c("A", "B"), G2 = c("C", "D", "E")))
get_groups(net)
}
\seealso{
\code{\link{set_groups}}, \code{\link{splot}}
}
