% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout-target-saqr.R
\name{layout-target-saqr}
\alias{layout-target-saqr}
\title{Target and Saqr Layouts}
\description{
Focal-node flow layouts: \code{target} (ported from qgraph's
\code{flow()}) and \code{saqr} (ported from the Dynalytics Desktop
transition-network viewer).
}
\keyword{internal}
