% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network-summary.R
\name{network_clique_size}
\alias{network_clique_size}
\title{Largest Clique Size}
\usage{
network_clique_size(x, ...)
}
\arguments{
\item{x}{Network input: matrix, igraph, network, cograph_network, or tna object}

\item{...}{Additional arguments passed to \code{\link{to_igraph}}}
}
\value{
Integer: size of the largest clique
}
\description{
Finds the size of the largest clique (complete subgraph) in the network.
Also known as the clique number or omega of the graph.
}
\examples{
# Triangle embedded in larger graph
adj <- matrix(c(0,1,1,1, 1,0,1,0, 1,1,0,0, 1,0,0,0), 4, 4)
network_clique_size(adj)  # 3
}
