% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster-metrics.R
\name{aggregate_weights}
\alias{aggregate_weights}
\alias{wagg}
\title{Aggregate Edge Weights}
\usage{
aggregate_weights(w, method = "sum", n_possible = NULL)

wagg(w, method = "sum", n_possible = NULL)
}
\arguments{
\item{w}{Numeric vector of edge weights}

\item{method}{Aggregation method: "sum", "mean", "median", "max", "min",
"prod", "density", "geomean"}

\item{n_possible}{Number of possible edges (for density calculation)}
}
\value{
Single aggregated value
}
\description{
Aggregates a vector of edge weights using various methods.
Compatible with igraph's edge.attr.comb parameter.
}
\examples{
w <- c(0.5, 0.8, 0.3, 0.9)
aggregate_weights(w, "sum")   # 2.5
aggregate_weights(w, "mean")  # 0.625
aggregate_weights(w, "max")   # 0.9
}
