% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster-metrics.R
\name{supra_interlayer}
\alias{supra_interlayer}
\alias{extract_interlayer}
\title{Extract Inter-Layer Block}
\usage{
supra_interlayer(x, from, to)

extract_interlayer(x, from, to)
}
\arguments{
\item{x}{Supra-adjacency matrix}

\item{from}{Source layer index}

\item{to}{Target layer index}
}
\value{
Inter-layer adjacency matrix
}
\description{
Extract Inter-Layer Block
}
\examples{
L1 <- matrix(c(0,.5,.3,.5,0,.4,.3,.4,0), 3, 3)
L2 <- matrix(c(0,.2,.6,.2,0,.1,.6,.1,0), 3, 3)
S <- supra_adjacency(list(L1 = L1, L2 = L2), omega = 0.5)
supra_interlayer(S, 1, 2)
L1 <- matrix(c(0,.5,.3,.5,0,.4,.3,.4,0), 3, 3)
L2 <- matrix(c(0,.2,.6,.2,0,.1,.6,.1,0), 3, 3)
S <- supra_adjacency(list(L1 = L1, L2 = L2), omega = 0.5)
extract_interlayer(S, 1, 2)
}
