% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/communities.R
\name{plot.cograph_communities}
\alias{plot.cograph_communities}
\title{Plot Community Structure}
\usage{
\method{plot}{cograph_communities}(x, network = NULL, ...)
}
\arguments{
\item{x}{A cograph_communities object}

\item{network}{The original network (required if not stored)}

\item{...}{Additional arguments passed to splot}
}
\value{
Invisibly returns the plot
}
\description{
Visualizes network with community coloring using splot.
}
\examples{
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
g <- igraph::make_graph("Zachary")
comm <- community_louvain(g)
mat <- igraph::as_adjacency_matrix(g, sparse = FALSE)
plot(comm, network = mat)
\dontshow{\}) # examplesIf}
}
