% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-globals.R
\name{list_layouts}
\alias{list_layouts}
\title{List Available Layouts}
\usage{
list_layouts()
}
\value{
Character vector of registered layout names.
}
\description{
List Available Layouts
}
\examples{
list_layouts()
}
