% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout-circle.R
\name{layout-circle}
\alias{layout-circle}
\title{Circular Layout}
\description{
Arrange nodes in a circle.
}
\keyword{internal}
