% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes-builtin.R
\name{themes-builtin}
\alias{themes-builtin}
\title{Built-in Themes}
\value{
A \code{CographTheme} object.
}
\description{
Pre-defined themes for network visualization.
}
\examples{
theme_cograph_classic()
theme_cograph_dark()
}
