% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input-statnet.R
\name{input-statnet}
\alias{input-statnet}
\title{Statnet Network Input Parsing}
\description{
Functions for parsing statnet network objects.
}
\keyword{internal}
