% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input-tna.R
\name{is_tna_network}
\alias{is_tna_network}
\title{Check if Network is TNA-based}
\usage{
is_tna_network(x)
}
\arguments{
\item{x}{A CographNetwork or cograph_network object.}
}
\value{
Logical: TRUE if the network was created from a TNA object, FALSE otherwise.
}
\description{
Checks whether a cograph_network was created from a tna or group_tna object.
}
\examples{
# Non-TNA network
mat <- matrix(c(0, 1, 1, 1, 0, 1, 1, 1, 0), nrow = 3)
net <- as_cograph(mat)
is_tna_network(net)  # FALSE

\dontshow{if (requireNamespace("tna", quietly = TRUE)) withAutoprint(\{ # examplesIf}
model <- tna::tna(tna::group_regulation)
net_tna <- as_cograph(model)
is_tna_network(net_tna)  # TRUE
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{as_cograph}}
}
