% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-print.R
\name{print.cograph_network}
\alias{print.cograph_network}
\title{Print cograph_network Object}
\usage{
\method{print}{cograph_network}(x, ...)
}
\arguments{
\item{x}{A cograph_network object.}

\item{...}{Ignored.}
}
\value{
The input object \code{x}, invisibly.
}
\description{
Print cograph_network Object
}
\examples{
adj <- matrix(c(0, 1, 1, 1, 0, 1, 1, 1, 0), nrow = 3)
net <- cograph(adj)
print(net)

}
