% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster-metrics.R
\name{plot.cograph_cluster_significance}
\alias{plot.cograph_cluster_significance}
\title{Plot Cluster Significance}
\usage{
\method{plot}{cograph_cluster_significance}(x, ...)
}
\arguments{
\item{x}{A \code{cograph_cluster_significance} object}

\item{...}{Additional arguments passed to \code{hist}}
}
\value{
Invisibly returns x
}
\description{
Creates a histogram of the null distribution with the observed value marked.
}
\examples{
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
g <- igraph::make_graph("Zachary")
comm <- community_louvain(g)
sig <- cluster_significance(g, comm, n_random = 20, seed = 42)
plot(sig)
\dontshow{\}) # examplesIf}
}
