% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-print.R
\name{methods-print}
\alias{methods-print}
\title{Print Methods}
\description{
S3 print methods for Cograph objects.
}
\keyword{internal}
