% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/communities.R
\name{community_leading_eigenvector}
\alias{community_leading_eigenvector}
\alias{com_le}
\title{Leading Eigenvector Community Detection}
\usage{
community_leading_eigenvector(
  x,
  weights = NULL,
  steps = -1,
  start = NULL,
  options = igraph::arpack_defaults(),
  callback = NULL,
  extra = NULL,
  env = parent.frame(),
  ...
)

com_le(
  x,
  weights = NULL,
  steps = -1,
  start = NULL,
  options = igraph::arpack_defaults(),
  callback = NULL,
  extra = NULL,
  env = parent.frame(),
  ...
)
}
\arguments{
\item{x}{Network input}

\item{weights}{Edge weights. NULL uses network weights, NA for unweighted.}

\item{steps}{Maximum number of splits. Default -1 (until modularity decreases).}

\item{start}{Starting community structure (membership vector).}

\item{options}{ARPACK options list. Default uses igraph::arpack_defaults().}

\item{callback}{Optional callback function called after each split.}

\item{extra}{Extra argument passed to callback.}

\item{env}{Environment for callback evaluation.}

\item{...}{Additional arguments passed to \code{\link{to_igraph}}}
}
\value{
A \code{cograph_communities} object

A \code{cograph_communities} object. See \code{\link{detect_communities}}.
}
\description{
Detects communities using the leading eigenvector of the modularity matrix.
Hierarchical divisive algorithm.
}
\examples{
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
g <- igraph::make_graph("Zachary")
comm <- community_leading_eigenvector(g)
membership(comm)
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
net <- as_cograph(matrix(runif(25), 5, 5))
com_le(net)
\dontshow{\}) # examplesIf}
}
\references{
Newman, M.E.J. (2006).
Finding community structure using the eigenvectors of matrices.
\emph{Physical Review E}, 74, 036104.
}
