% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motifs.R
\name{get_edge_list}
\alias{get_edge_list}
\title{Extract Raw Edge List from TNA Model}
\usage{
get_edge_list(x, by_individual = TRUE, drop_zeros = TRUE)
}
\arguments{
\item{x}{A tna object created by \code{\link[tna:build_model]{tna::tna()}}}

\item{by_individual}{Logical. If TRUE (default), returns edge list with
individual IDs. If FALSE, aggregates across all individuals.}

\item{drop_zeros}{Logical. If TRUE (default), excludes edges with zero count.}
}
\value{
A data frame with columns:
\describe{
\item{id}{Individual identifier (only if \code{by_individual = TRUE})}
\item{from}{Source state label}
\item{to}{Target state label}
\item{count}{Number of transitions}
}
}
\description{
Extract individual-level transition counts as an edge list from a tna object.
}
\examples{
\dontshow{if (requireNamespace("tna", quietly = TRUE)) withAutoprint(\{ # examplesIf}
Mod <- tna::tna(tna::group_regulation)

# Get edge list by individual
edges <- get_edge_list(Mod)
head(edges)

# Aggregate across individuals
agg_edges <- get_edge_list(Mod, by_individual = FALSE)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link[=extract_motifs]{extract_motifs()}} for motif analysis using edge lists

Other motifs: 
\code{\link{extract_motifs}()},
\code{\link{extract_triads}()},
\code{\link{motif_census}()},
\code{\link{motifs}()},
\code{\link{plot.cograph_motif_analysis}()},
\code{\link{plot.cograph_motifs}()},
\code{\link{subgraphs}()},
\code{\link{triad_census}()}
}
\concept{motifs}
