% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes-builtin.R
\name{theme_cograph_colorblind}
\alias{theme_cograph_colorblind}
\title{Colorblind-friendly Theme}
\usage{
theme_cograph_colorblind()
}
\value{
A CographTheme object.
}
\description{
Theme using colors distinguishable by people with color vision deficiency.
}
\examples{
theme <- theme_cograph_colorblind()
}
