% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/communities.R
\name{community_sizes}
\alias{community_sizes}
\title{Get Community Sizes}
\usage{
community_sizes(x)
}
\arguments{
\item{x}{A cograph_communities object}
}
\value{
Integer vector of community sizes
}
\description{
Get Community Sizes
}
\examples{
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
g <- igraph::make_graph("Zachary")
comm <- community_louvain(g)
community_sizes(comm)
\dontshow{\}) # examplesIf}
}
