% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes-builtin.R
\name{theme_cograph_classic}
\alias{theme_cograph_classic}
\title{Classic Theme}
\usage{
theme_cograph_classic()
}
\value{
A CographTheme object.
}
\description{
Traditional network visualization style with blue nodes and gray edges.
}
\examples{
theme <- theme_cograph_classic()
}
