% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/communities.R
\name{community_edge_betweenness}
\alias{community_edge_betweenness}
\alias{com_eb}
\title{Edge Betweenness Community Detection}
\usage{
community_edge_betweenness(
  x,
  weights = NULL,
  directed = TRUE,
  edge.betweenness = TRUE,
  merges = TRUE,
  bridges = TRUE,
  modularity = TRUE,
  membership = TRUE,
  ...
)

com_eb(
  x,
  weights = NULL,
  directed = TRUE,
  edge.betweenness = TRUE,
  merges = TRUE,
  bridges = TRUE,
  modularity = TRUE,
  membership = TRUE,
  ...
)
}
\arguments{
\item{x}{Network input}

\item{weights}{Edge weights. NULL uses network weights, NA for unweighted.}

\item{directed}{Logical; treat graph as directed? Default TRUE.}

\item{edge.betweenness}{Logical; return edge betweenness values? Default TRUE.}

\item{merges}{Logical; return merge matrix? Default TRUE.}

\item{bridges}{Logical; return bridge edges? Default TRUE.}

\item{modularity}{Logical; return modularity scores? Default TRUE.}

\item{membership}{Logical; return membership vector? Default TRUE.}

\item{...}{Additional arguments passed to \code{\link{to_igraph}}}
}
\value{
A \code{cograph_communities} object

A \code{cograph_communities} object. See \code{\link{detect_communities}}.
}
\description{
Girvan-Newman algorithm. Iteratively removes edges with highest
betweenness centrality to reveal community structure.
}
\examples{
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
g <- igraph::make_graph("Zachary")
comm <- community_edge_betweenness(g)
membership(comm)
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
net <- as_cograph(matrix(runif(25), 5, 5))
com_eb(net)
\dontshow{\}) # examplesIf}
}
\references{
Girvan, M., & Newman, M.E.J. (2002).
Community structure in social and biological networks.
\emph{PNAS}, 99(12), 7821-7826.
}
