% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-nestimate.R
\name{plot_net_stability}
\alias{plot_net_stability}
\title{Plot Centrality Stability Results}
\usage{
plot_net_stability(x, ...)
}
\arguments{
\item{x}{A \code{net_stability} object (from \code{Nestimate::centrality_stability}).}

\item{...}{Additional graphical arguments.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Visualizes the centrality stability analysis from a \code{net_stability}
object. Shows how centrality correlations drop as cases are removed.
}
\examples{
\dontshow{if (requireNamespace("Nestimate", quietly = TRUE)) withAutoprint(\{ # examplesIf}
set.seed(1)
seqs <- data.frame(T1 = sample(c("A","B","C"), 30, replace = TRUE),
                   T2 = sample(c("A","B","C"), 30, replace = TRUE))
net <- Nestimate::build_network(seqs, method = "tna")
cs <- Nestimate::centrality_stability(net, iter = 10)
plot_net_stability(cs)
\dontshow{\}) # examplesIf}
}
