% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-globals.R
\name{list_themes}
\alias{list_themes}
\title{List Available Themes}
\usage{
list_themes()
}
\value{
Character vector of registered theme names.
}
\description{
List Available Themes
}
\examples{
list_themes()
}
