% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes-builtin.R
\name{theme_cograph_nature}
\alias{theme_cograph_nature}
\title{Nature Theme}
\usage{
theme_cograph_nature()
}
\value{
A CographTheme object.
}
\description{
Earth tones theme inspired by nature.
}
\examples{
theme <- theme_cograph_nature()
}
