% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shapes-svg.R
\name{list_svg_shapes}
\alias{list_svg_shapes}
\title{List Registered SVG Shapes}
\usage{
list_svg_shapes()
}
\value{
Character vector of registered shape names.
}
\description{
Get names of all registered custom SVG shapes.
}
\examples{
list_svg_shapes()
}
