% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-plot.R
\name{summary.cograph_network}
\alias{summary.cograph_network}
\title{Summary of cograph_network Object}
\usage{
\method{summary}{cograph_network}(object, ...)
}
\arguments{
\item{object}{A cograph_network object.}

\item{...}{Ignored.}
}
\value{
A list with network summary information (invisibly), containing
elements \code{n_nodes}, \code{n_edges}, \code{directed}, \code{weighted},
and \code{has_layout}.
}
\description{
Summary of cograph_network Object
}
\examples{
adj <- matrix(c(0, 1, 1, 1, 0, 1, 1, 1, 0), nrow = 3)
net <- cograph(adj)
summary(net)

}
