% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{palettes}
\alias{palettes}
\title{Color Palettes}
\description{
Built-in color palettes for network visualization.
}
\examples{
palette_blues(5)
palette_reds(5)
}
