% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/communities.R
\name{print.cograph_communities}
\alias{print.cograph_communities}
\title{Print Community Structure}
\usage{
\method{print}{cograph_communities}(x, ...)
}
\arguments{
\item{x}{A cograph_communities object.}

\item{...}{Ignored.}
}
\value{
Invisibly returns the original object.
}
\description{
Print Community Structure
}
\examples{
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
g <- igraph::make_graph("Zachary")
comm <- community_louvain(g)
print(comm)
\dontshow{\}) # examplesIf}
}
