% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-network.R
\name{get_edges}
\alias{get_edges}
\title{Get Edges from Cograph Network}
\usage{
get_edges(x)
}
\arguments{
\item{x}{A cograph_network object.}
}
\value{
A data frame with columns: from, to, weight.
}
\description{
Extracts the edges data frame from a cograph_network object.
}
\examples{
mat <- matrix(c(0, 1, 1, 1, 0, 1, 1, 1, 0), nrow = 3)
net <- as_cograph(mat)
get_edges(net)
}
\seealso{
\code{\link{as_cograph}}, \code{\link{n_edges}}, \code{\link{get_nodes}}
}
