% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robustness.R
\name{robustness_summary}
\alias{robustness_summary}
\title{Summary of Robustness Analysis}
\usage{
robustness_summary(..., x = NULL, measures = NULL, n_iter = 1000)
}
\arguments{
\item{...}{Robustness results to summarize.}

\item{x}{Network for on-the-fly computation.}

\item{measures}{Measures to compute if x provided.}

\item{n_iter}{Iterations for random. Default 1000.}
}
\value{
Data frame with AUC and critical points for each measure.
}
\description{
Provides a summary comparing robustness metrics across attack strategies.
}
\examples{
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
g <- igraph::sample_pa(30, m = 2, directed = FALSE)
robustness_summary(x = g, measures = c("degree", "random"), n_iter = 10)
\dontshow{\}) # examplesIf}
}
