% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout-oval.R
\name{layout-oval}
\alias{layout-oval}
\title{Oval/Ellipse Layout}
\description{
Arrange nodes in an oval (ellipse) shape.
}
\keyword{internal}
